==> jstrack-3.0.2.tbz: lib/jstrack/ftpfilt/ftplib.tcl <==

# JStrack:  copyright 1997/2008 by Jim Graham, N5IAL, all rights reserved.

# If you have firewall troubles (e.g., ftp just won't transfer files,
# won't connect, etc.), try using passive mode instead.  Just change
# the '0' (zero) to a '1' (one) in the following:

set passive_mode_ftp 0

# DON'T TOUCH THESE UNLESS YOU KNOW EXACTLY WHAT YOU'RE DOING!!!
set standalone_ftp_mode 0
set nhcsite ftp.nhc.noaa.gov
set nhctopdir /pub/products/nhc
set subdirs [list marine strike disc public]

source /usr/local/lib/jstrack/paths.tcl


proc usrmsg {msg {statwin 0}} {
   global notk
#  set f [open ftplog a] ; puts $f $msg ; flush $f ; close $f
   if {$notk} {
      if {$statwin == 0} { puts $msg }
   } else {
      set stlist [list jdgmsg statmsg errmsg crashmsg]
      .ftpmsg.text insert end "$msg\n" [lindex $stlist $statwin]
      .ftpmsg.text see end
      update idletasks
   }
}


proc tk_finish {} {
   global notk standalone_ftp_mode
   if {$notk && $standalone_ftp_mode} { exit }
   if {$notk} { return }
   if {$standalone_ftp_mode} {
      button .ftpmsg.done -text "Dismiss" -command { exit }
   } else {
      button .ftpmsg.done -text "Dismiss" -command { destroy .ftpmsg }
   }
   pack .ftpmsg.done -side top
}


proc noftplib {msg} {
  global ftp_available notk
   set ftp_available 0

   catch { destroy .ftpmsg }
   if {[catch {toplevel .ftpmsg}]} {
      set notk 1
   } else {
      set notk 0
      text .ftpmsg.text -relief sunken -bd 2 -width 80 -height 25 \
         -yscrollcommand ".ftpmsg.scroll set" -font {Courier 12}
      scrollbar .ftpmsg.scroll -command ".ftpmsg.text yview"
      pack .ftpmsg.scroll -side right -fill y
      pack .ftpmsg.text -side top -expand yes -fill both
   }

   usrmsg $msg
   if {$notk == 0} { tk_finish }
}


proc ftpmsg {s msg {state ""}} {
   global closing_ftp ftpcrash

   set lvl 1 ;# not an error
   if {[regexp {C: [45]} $msg]} {
      if {$closing_ftp == 0 && [regexp {C: [45]21} $msg]} { set ftpcrash 1 }
      set lvl 2 ;# error
   } elseif {[regexp {C: 221} $msg]} { set ftpcrash 1 }
   if {$state == "error"} { set lvl 2 } ;# error
   usrmsg $msg $lvl
}


proc openftp {ftpsite {username anonymous} {passwd ftpuser@domain}} {
   global ftpcrash passive_mode_ftp
   if {$ftpcrash} { return "" }
   if {$passive_mode_ftp} {
      return [::ftp::Open $ftpsite $username $passwd -mode passive \
         -output ftpmsg]
   }
   return [::ftp::Open $ftpsite $username $passwd -output ftpmsg]
}
proc ftpcd {f dir} {
   global ftpcrash ; if {$ftpcrash} { return }
   ::ftp::Cd $f $dir
}
proc getls {f {pattern ""}} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   return [::ftp::NList $f $pattern]
}
proc ftpf_tm {f fn} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   return [::ftp::ModTime $f $fn]
}
proc closeftp {f} {
   global ftpcrash ; if {$ftpcrash} { return }
   usrmsg "Finished!  Closing FTP connection." 3
   after 1000 ;
   ::ftp::Close $f
}
proc ftp_mode {f mode} {
   global ftpcrash ; if {$ftpcrash} { return }
   ::ftp::Type $f $mode
}
proc ftp_get {f fn} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   ::ftp::Get $f $fn -variable ret
   if {$ftpcrash} { return "" }
   return $ret
}


proc sortlist {f l dir} {
   global ftpdir
   if {[file exists $ftpdir/timestamp.$dir]} {
      set last_tm [file mtime $ftpdir/timestamp.$dir]
   } else {
      set last_tm 0
   }
   set lastftp [clock format $last_tm -format %Y%m%d%H -gmt true]
   set r1 [list]
   foreach fn $l {
      set tm [file rootname $fn]
      if {$tm > $lastftp} {
         lappend r1 $fn
      } elseif {$tm == $lastftp} {
         if {[ftpf_tm $f $fn] > $last_tm} { lappend r1 $fn }
      }
   }

#  This was a BAD idea...assuming that the ftp transfers would go
#  smoothly, and never crash ... WTF was I thinking?
#
#  set ftmp [open $ftpdir/timestamp.$dir w]
#  close $ftmp
   return [lsort $r1]
}


proc proc_ftpcrash {} { tk_finish }


proc update_ts_file {dir tm} {
   global ftpdir
   set fn $ftpdir/timestamp.$dir
   if {![file exists $fn]} { set ftmp [open $fn w] ; close $ftmp }
   ::fileutil::touch -c -t $tm $fn
# usrmsg "set timestamp on $fn to [clock format $tm]"
}


# Main body of code...wasn't a procedure before, but now that it's
# called by JStrack, it needs to be able to return...not exit!

proc update_ftp {} {
   global nhcsite nhctopdir subdirs tclshbin filt_path notk
   global standalone_ftp_mode ftp_available closing_ftp ftpcrash
   global tcl_platform

   if {$ftp_available == 0} { return }
   set closing_ftp 0
   set ftpcrash 0

   catch { destroy .ftpmsg }
   if {[catch {toplevel .ftpmsg}]} {
      set notk 1
   } else {
      set notk 0
      text .ftpmsg.text -relief sunken -bd 2 -width 80 -height 25 \
         -yscrollcommand ".ftpmsg.scroll set" -font {Courier 12}
      scrollbar .ftpmsg.scroll -command ".ftpmsg.text yview"
      pack .ftpmsg.scroll -side right -fill y
      pack .ftpmsg.text -side top -expand yes -fill both
      .ftpmsg.text tag configure jdgmsg -font {Courier 10 bold} -foreground #0000dd
      .ftpmsg.text tag configure crashmsg -font {Courier 14 bold} -foreground #0000dd
      .ftpmsg.text tag configure errmsg -font {Courier 10 bold} -foreground #dd0000
      .ftpmsg.text tag configure statmsg -font {Courier 10} -foreground #000000
      if {$standalone_ftp_mode} { wm withdraw . }
   }

   usrmsg "Opening connection to site..."
   if {[catch { set username $tcl_platform(user) }]} { set username user }
   if {[catch { set hostname [info hostname] }]} { set hostname host }
   set f [openftp $nhcsite anonymous $username@$hostname.somedomain.com]
   if {$f < 0} {
      usrmsg "could not open $nhcsite ... aborting"
      if {$standalone_ftp_mode && $notk} {
         exit 1
      } else {
         tk_finish
      }
   }

   foreach subdir $subdirs {
      if {$ftpcrash} { proc_ftpcrash ; return }
      ftpcd $f $nhctopdir/$subdir
      if {$ftpcrash} { proc_ftpcrash ; return }
      usrmsg $nhctopdir/$subdir....
      set fnlist [sortlist $f [getls $f *.atl] $subdir]
      if {$ftpcrash} { proc_ftpcrash ; return }

      foreach fn $fnlist {
         usrmsg "Grabbing $fn"
         set inadv 0

         ftp_mode $f ascii
         if {$ftpcrash} { proc_ftpcrash ; return }
         set ret [ftp_get $f $fn]
         if {$ftpcrash} { proc_ftpcrash ; return }
         set fnts [ftpf_tm $f $fn]
         if {$ftpcrash} { proc_ftpcrash ; return }

# Original ... before fixing file transfer mode
#        set rlist [textutil::splitx $ret {*}]

         if {[regexp {KNHC[^\n\r]*([\n\r][\n\r]*)[^\n\r]} $ret a linesep]} {
usrmsg "Line sep. found in $fn ... processing."
            regsub -all $linesep $ret {\n} t
            set rlist [textutil::splitx $t {\\n}]
         } else {
            # use this and hope it works
usrmsg "Line sep. not found in $fn ... proceeding with fingers crossed"
            set rlist [textutil::splitx $ret {[\\r\\n]+}]
         }

         foreach line $rlist {
            regsub -all {} $line {} t
            regsub -all {} $t {} t2
            regsub -all {} $t2 {} t3
            set line [string trimleft [string trimright $t3]]
#et ftest [open $fn.fixed w]
#oreach line $rlist { puts $ftest $line }
#lose $ftest

            if {[regexp KNHC $line] || [regexp KMIA $line]} {
               if {$inadv} {
                  close $fout
                  usrmsg "   processing data..."
                  catch { exec $tclshbin $filt_path < tmp_ftp$f.txt }
# Yes, the timestamp may be set more than once per file....
# This way, though, it will only be set if we've got data....
                  update_ts_file $subdir $fnts
                  file delete -force tmp_ftp$f.txt
               }
               set fout [open tmp_ftp$f.txt w]
               set inadv 1
            }
            if {$inadv} { puts $fout $line }
         }
         if {$inadv} {
            close $fout
            usrmsg "   processing data..."
            catch { exec $tclshbin $filt_path < tmp_ftp$f.txt }
# Yes, the timestamp may be set more than once per file....
# This way, though, it will only be set if we've got data....
            update_ts_file $subdir $fnts
            file delete -force tmp_ftp$f.txt
         }
      }
   }

   if {$ftpcrash} { proc_ftpcrash ; return }
   set closing_ftp 1
   closeftp $f
   usrmsg "DONE"
   if {$notk == 0} { tk_finish }
}


set ftp_available 1
if {[catch { package require Tcl 8.2 }]}  { noftplib "Need at least Tcl 8.2 for tcllib!" }
if {[catch { package require ftp 2.3.1 }]}      { noftplib "Need tcllib 1.3 or better!" }
if {[catch { package require textutil }]} { noftplib "Need tcllib 1.3 or better!" }
if {[catch { package require fileutil }]} { noftplib "Need tcllib 1.3 or better!" }
set ::ftp::VERBOSE 1

==> jstrack-3.1.0.tbz: lib/jstrack/ftpfilt/ftplib.tcl <==

# JStrack:  copyright 1997/2008 by Jim Graham, N5IAL, all rights reserved.

# If you have firewall troubles (e.g., ftp just won't transfer files,
# won't connect, etc.), try using passive mode instead.  Just change
# the '0' (zero) to a '1' (one) in the following:

set passive_mode_ftp 0

# DON'T TOUCH THESE UNLESS YOU KNOW EXACTLY WHAT YOU'RE DOING!!!
set standalone_ftp_mode 0
set nhcsite ftp.nhc.noaa.gov
set nhctopdir /pub/products/nhc
set subdirs [list marine disc public]

source ../paths.tcl


proc usrmsg {msg {statwin 0}} {
   global notk
#  set f [open ftplog a] ; puts $f $msg ; flush $f ; close $f
   if {$notk} {
      if {$statwin == 0} { puts $msg }
   } else {
      set stlist [list jdgmsg statmsg errmsg crashmsg]
      .ftpmsg.text insert end "$msg\n" [lindex $stlist $statwin]
      .ftpmsg.text see end
      update idletasks
   }
}


proc tk_finish {} {
   global notk standalone_ftp_mode
   if {$notk && $standalone_ftp_mode} { exit }
   if {$notk} { return }
   if {$standalone_ftp_mode} {
      button .ftpmsg.done -text "Dismiss" -command { exit }
   } else {
      button .ftpmsg.done -text "Dismiss" -command { destroy .ftpmsg }
   }
   pack .ftpmsg.done -side top
}


proc noftplib {msg} {
  global ftp_available notk
   set ftp_available 0

   catch { destroy .ftpmsg }
   if {[catch {toplevel .ftpmsg}]} {
      set notk 1
   } else {
      set notk 0
      text .ftpmsg.text -relief sunken -bd 2 -width 80 -height 25 \
         -yscrollcommand ".ftpmsg.scroll set" -font {Courier 12}
      scrollbar .ftpmsg.scroll -command ".ftpmsg.text yview"
      pack .ftpmsg.scroll -side right -fill y
      pack .ftpmsg.text -side top -expand yes -fill both
   }

   usrmsg $msg
   if {$notk == 0} { tk_finish }
}


proc ftpmsg {s msg {state ""}} {
   global closing_ftp ftpcrash

   set lvl 1 ;# not an error
   if {[regexp {C: [45]} $msg]} {
      if {$closing_ftp == 0 && [regexp {C: [45]21} $msg]} { set ftpcrash 1 }
      set lvl 2 ;# error
   } elseif {[regexp {C: 221} $msg]} { set ftpcrash 1 }
   if {$state == "error"} { set lvl 2 } ;# error
   usrmsg $msg $lvl
}


proc openftp {ftpsite {username anonymous} {passwd ftpuser@domain}} {
   global ftpcrash passive_mode_ftp
   if {$ftpcrash} { return "" }
   if {$passive_mode_ftp} {
      return [::ftp::Open $ftpsite $username $passwd -mode passive \
         -output ftpmsg]
   }
   return [::ftp::Open $ftpsite $username $passwd -output ftpmsg]
}
proc ftpcd {f dir} {
   global ftpcrash ; if {$ftpcrash} { return }
   ::ftp::Cd $f $dir
}
proc getls {f {pattern ""}} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   return [::ftp::NList $f $pattern]
}
proc ftpf_tm {f fn} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   return [::ftp::ModTime $f $fn]
}
proc closeftp {f} {
   global ftpcrash ; if {$ftpcrash} { return }
   usrmsg "Finished!  Closing FTP connection." 3
   after 1000 ;
   ::ftp::Close $f
}
proc ftp_mode {f mode} {
   global ftpcrash ; if {$ftpcrash} { return }
   ::ftp::Type $f $mode
}
proc ftp_get {f fn} {
   global ftpcrash ; if {$ftpcrash} { return "" }
   ::ftp::Get $f $fn -variable ret
   if {$ftpcrash} { return "" }
   return $ret
}


proc sortlist {f l dir} {
   global ftpdir
   if {[file exists $ftpdir/timestamp.$dir]} {
      set last_tm [file mtime $ftpdir/timestamp.$dir]
   } else {
      set last_tm 0
   }
   set lastftp [clock format $last_tm -format %Y%m%d%H -gmt true]
   set r1 [list]
   foreach fn $l {
      set tm [file rootname $fn]
      if {$tm > $lastftp} {
         lappend r1 $fn
      } elseif {$tm == $lastftp} {
         if {[ftpf_tm $f $fn] > $last_tm} { lappend r1 $fn }
      }
   }

#  This was a BAD idea...assuming that the ftp transfers would go
#  smoothly, and never crash ... WTF was I thinking?
#
#  set ftmp [open $ftpdir/timestamp.$dir w]
#  close $ftmp
   return [lsort $r1]
}


proc proc_ftpcrash {} { tk_finish }


proc update_ts_file {dir tm} {
   global ftpdir
   set fn $ftpdir/timestamp.$dir
   if {![file exists $fn]} { set ftmp [open $fn w] ; close $ftmp }
   ::fileutil::touch -c -t $tm $fn
# usrmsg "set timestamp on $fn to [clock format $tm]"
}


# Main body of code...wasn't a procedure before, but now that it's
# called by JStrack, it needs to be able to return...not exit!

proc update_ftp {} {
   global nhcsite nhctopdir subdirs tclshbin filt_path notk
   global standalone_ftp_mode ftp_available closing_ftp ftpcrash
   global tcl_platform

   if {$ftp_available == 0} { return }
   set closing_ftp 0
   set ftpcrash 0

   catch { destroy .ftpmsg }
   if {[catch {toplevel .ftpmsg}]} {
      set notk 1
   } else {
      set notk 0
      text .ftpmsg.text -relief sunken -bd 2 -width 80 -height 25 \
         -yscrollcommand ".ftpmsg.scroll set" -font {Courier 12}
      scrollbar .ftpmsg.scroll -command ".ftpmsg.text yview"
      pack .ftpmsg.scroll -side right -fill y
      pack .ftpmsg.text -side top -expand yes -fill both
      .ftpmsg.text tag configure jdgmsg -font {Courier 10 bold} -foreground #0000dd
      .ftpmsg.text tag configure crashmsg -font {Courier 14 bold} -foreground #0000dd
      .ftpmsg.text tag configure errmsg -font {Courier 10 bold} -foreground #dd0000
      .ftpmsg.text tag configure statmsg -font {Courier 10} -foreground #000000
      if {$standalone_ftp_mode} { wm withdraw . }
   }

   usrmsg "Opening connection to site..."
   if {[catch { set username $tcl_platform(user) }]} { set username user }
   if {[catch { set hostname [info hostname] }]} { set hostname host }
   set f [openftp $nhcsite anonymous $username@$hostname.somedomain.com]
   if {$f < 0} {
      usrmsg "could not open $nhcsite ... aborting"
      if {$standalone_ftp_mode && $notk} {
         exit 1
      } else {
         tk_finish
      }
   }

   foreach subdir $subdirs {
      if {$ftpcrash} { proc_ftpcrash ; return }
      ftpcd $f $nhctopdir/$subdir
      if {$ftpcrash} { proc_ftpcrash ; return }
      usrmsg $nhctopdir/$subdir....
      set fnlist [sortlist $f [getls $f *.atl] $subdir]
      if {$ftpcrash} { proc_ftpcrash ; return }

      foreach fn $fnlist {
         usrmsg "Grabbing $fn"
         set inadv 0

         ftp_mode $f ascii
         if {$ftpcrash} { proc_ftpcrash ; return }
         set ret [ftp_get $f $fn]
         if {$ftpcrash} { proc_ftpcrash ; return }
         set fnts [ftpf_tm $f $fn]
         if {$ftpcrash} { proc_ftpcrash ; return }

# Original ... before fixing file transfer mode
#        set rlist [textutil::splitx $ret {*}]

         if {[regexp {KNHC[^\n\r]*([\n\r][\n\r]*)[^\n\r]} $ret a linesep]} {
usrmsg "Line sep. found in $fn ... processing."
            regsub -all $linesep $ret {\n} t
            set rlist [textutil::splitx $t {\\n}]
         } else {
            # use this and hope it works
usrmsg "Line sep. not found in $fn ... proceeding with fingers crossed"
            set rlist [textutil::splitx $ret {[\\r\\n]+}]
         }

         foreach line $rlist {
            regsub -all {} $line {} t
            regsub -all {} $t {} t2
            regsub -all {} $t2 {} t3
            set line [string trimleft [string trimright $t3]]
#et ftest [open $fn.fixed w]
#oreach line $rlist { puts $ftest $line }
#lose $ftest

            if {[regexp KNHC $line] || [regexp KMIA $line]} {
               if {$inadv} {
                  close $fout
                  usrmsg "   processing data..."
                  catch { exec $tclshbin $filt_path < tmp_ftp$f.txt }
# Yes, the timestamp may be set more than once per file....
# This way, though, it will only be set if we've got data....
                  update_ts_file $subdir $fnts
                  file delete -force tmp_ftp$f.txt
               }
               set fout [open tmp_ftp$f.txt w]
               set inadv 1
            }
            if {$inadv} { puts $fout $line }
         }
         if {$inadv} {
            close $fout
            usrmsg "   processing data..."
            catch { exec $tclshbin $filt_path < tmp_ftp$f.txt }
# Yes, the timestamp may be set more than once per file....
# This way, though, it will only be set if we've got data....
            update_ts_file $subdir $fnts
            file delete -force tmp_ftp$f.txt
         }
      }
   }

   if {$ftpcrash} { proc_ftpcrash ; return }
   set closing_ftp 1
   closeftp $f
   usrmsg "DONE"
   if {$notk == 0} { tk_finish }
}


set ftp_available 1
if {[catch { package require Tcl 8.2 }]}  { noftplib "Need at least Tcl 8.2 for tcllib!" }
if {[catch { package require ftp 2.3.1 }]}      { noftplib "Need tcllib 1.3 or better!" }
if {[catch { package require textutil }]} { noftplib "Need tcllib 1.3 or better!" }
if {[catch { package require fileutil }]} { noftplib "Need tcllib 1.3 or better!" }
set ::ftp::VERBOSE 1
